﻿using System;

namespace DecoratedTeacher
{
    public class ThreadSafeTeacher : ITeacher
    {
        private readonly ITeacher _theRealTeacher;
        private readonly Object _lockObject = new object();

        public ThreadSafeTeacher(ITeacher theRealTeacher)
        {
            if (theRealTeacher == null) throw new ArgumentNullException("theRealTeacher");
            _theRealTeacher = theRealTeacher;
        }

        public string Name
        {
            get { lock (_lockObject) { return _theRealTeacher.Name; } }
            set { lock (_lockObject) { _theRealTeacher.Name = value; } }
        }

        public int Salary
        {
            get { lock (_lockObject) { return _theRealTeacher.Salary; } }
            set { lock (_lockObject) { _theRealTeacher.Salary = value; } }
        }

        public bool Equals(ITeacher other)
        {
            lock (_lockObject)
            {
                return _theRealTeacher.Equals(other);
            }
        }

        public override bool Equals(Object obj)
        {
            lock (_lockObject)
            {
                return _theRealTeacher.Equals(obj);
            }
        }

        public override string ToString()
        {
            lock (_lockObject)
            {
                return _theRealTeacher.ToString();
            }
        }

        public override int GetHashCode()
        {
            lock (_lockObject)
            {
                return _theRealTeacher.GetHashCode();
            }
        }
    }
}
