﻿using System;
using DecoratedTeacher;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DecoratedTeacherTests
{
    [TestClass]
    public class ThreadSafeTeacherTests
    {
        private ITeacher _ordinaryTeacher;
        private ITeacher _threadSafeTeacher;

        [TestInitialize]
        public void Initialize()
        {
            _ordinaryTeacher = new Teacher { Name = "Anders", Salary = 1234 };
            _threadSafeTeacher = new ThreadSafeTeacher(_ordinaryTeacher);
        }

        [TestMethod]
        public void ThreadSafeTeacherTest()
        {
            try
            {
                new ThreadSafeTeacher(null);
                Assert.Fail();
            }
            catch (ArgumentNullException) { }
        }

        [TestMethod]
        public void NameTest()
        {
            Assert.AreEqual("Anders", _threadSafeTeacher.Name);
            _threadSafeTeacher.Name = "John";
            Assert.AreEqual("John", _ordinaryTeacher.Name);
        }

        [TestMethod]
        public void SalaryTest()
        {
            Assert.AreEqual(1234, _threadSafeTeacher.Salary);
            _threadSafeTeacher.Salary = 9999;
            Assert.AreEqual(9999, _ordinaryTeacher.Salary);
        }

        [TestMethod]
        public void EqualsTest()
        {
            Assert.AreEqual(_threadSafeTeacher, _threadSafeTeacher);
            Assert.AreEqual(_ordinaryTeacher, _threadSafeTeacher);
            Assert.AreEqual(_threadSafeTeacher, _ordinaryTeacher);
            Assert.IsTrue(_threadSafeTeacher.Equals(_ordinaryTeacher));
        }

        [TestMethod]
        public void ToStringTest()
        {
            Assert.AreEqual(_ordinaryTeacher.ToString(), _threadSafeTeacher.ToString());
        }

        [TestMethod]
        public void GetHashCodeTest()
        {
            Assert.AreEqual(_ordinaryTeacher.GetHashCode(), _threadSafeTeacher.GetHashCode());
        }
    }
}
