﻿using System;
using DecoratedTeacher;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DecoratedTeacherTests
{
    [TestClass]
    public class UnmodifiableTeacherTests
    {
        private ITeacher _ordinaryTeacher;
        private ITeacher _unmodifiableTeacher;

        [TestInitialize]
        public void Initialize()
        {
            _ordinaryTeacher = new Teacher { Name = "Anders", Salary = 1234 };
            _unmodifiableTeacher = new UnmodifiableTeacher(_ordinaryTeacher);
        }

        [TestMethod]
        public void UnmodifiableTeacherTest()
        {
            try
            {
                new UnmodifiableTeacher(null);
                Assert.Fail();
            }
            catch (ArgumentNullException) { }
        }


        [TestMethod]
        public void NameTest()
        {
            Assert.AreEqual("Anders", _unmodifiableTeacher.Name);
            try
            {
                _unmodifiableTeacher.Name = "John";
                Assert.Fail();
            }
            catch (NotSupportedException) { }
        }

        [TestMethod]
        public void SalaryTest()
        {
            Assert.AreEqual(1234, _unmodifiableTeacher.Salary);
            try
            {
                _unmodifiableTeacher.Salary = 9999;
                Assert.Fail();
            }
            catch (NotSupportedException) { }
        }

        [TestMethod]
        public void EqualsTest()
        {
            Assert.AreEqual(_unmodifiableTeacher, _unmodifiableTeacher);
            Assert.AreEqual(_ordinaryTeacher, _unmodifiableTeacher);
            Assert.AreEqual(_unmodifiableTeacher, _ordinaryTeacher);
            Assert.IsTrue(_unmodifiableTeacher.Equals(_ordinaryTeacher));
        }

        [TestMethod]
        public void ToStringTest()
        {
            Assert.AreEqual(_ordinaryTeacher.ToString(), _unmodifiableTeacher.ToString());
        }

        [TestMethod]
        public void GetHashCodeTest()
        {
            Assert.AreEqual(_ordinaryTeacher.GetHashCode(), _unmodifiableTeacher.GetHashCode());

        }
    }
}
